<?php 
/*
Plugin Name: WP e-Commerce S3 Downloads
Plugin URI: http://www.getshopped.co.nz
Description: Provides the ability to serve downloadable files through Amazon S3
Version: 1.0
Author: Instinct Entertainment
Author URI: www.getshopped.org
*/
require_once('S3.php');

function wpsc_s3_add_settings_page($page_hooks, $base_page) {
	$page_hooks[] = add_submenu_page($base_page,__('- S3 Downloads', 'wpsc'), __('- S3 Downloads', 'wpsc'), 9, 'wpsc-s3-configuration', 'wpsc_s3_settings_page');
	return $page_hooks;
}

add_filter('wpsc_additional_pages', 'wpsc_s3_add_settings_page', 10, 2);
// here to line 94 handles the settings page. Getting the info into the options table
function wpsc_s3_settings_page() {
	global $wpdb;
	$wpsc_s3_access_key = get_option('wpsc_s3_access_key');
	$wpsc_s3_secret_key = get_option('wpsc_s3_secret_key');
	$wpsc_s3_settings = get_option('wpsc_s3_settings'); ?>
	<div class="wrap">
	<div id="icon-options-general" class="icon32"><br /></div>
	<h2><?php _e("S3 Downloads",'wpsc'); ?></h2>
	<form action="" method="post" accept-charset="utf-8">

		<h3><?php _e("S3 Credentials",'wpsc'); ?></h3>
		<table class="form-table">
			<tr valign="top">
				<th scope="row"><?php _e('Access Key', 'wpsc'); ?></th>
				<td>
					<input type='text' name='access_key' value='<?php echo $wpsc_s3_access_key; ?>' size="59" class='regular-text' />
					<span class="description">Your Access Key acts as your 'username'</span>
				</td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php _e('Secret Key', 'wpsc'); ?></th>
				<td>
					<input type='text' name='secret_key' value='<?php echo $wpsc_s3_secret_key; ?>' size="59" class='regular-text' />
					<span class="description">Your Secret Key acts as your 'password'</span>
				</td>
			</tr>
		</table>

		<h3><?php _e("S3 Downloadable File Settings",'wpsc'); ?></h3>
		<table class="form-table">
				<tr valign="top">
				<th scope="row"><?php _e('S3 Bucket', 'wpsc'); ?></th>
				<td>
					<input type='text' name='s3_settings[bucket]' value='<?php echo $wpsc_s3_settings['bucket']; ?>' size="59" class='regular-text' /><br />
					<span class="description"><?php _e('Provide your unique bucket name, if the bucket does not exist it will be automatically created', 'wpsc'); ?></span>
				</td>
			</tr>
		</table>
	<br/>
	<?php // check if s3 works with provided details
	update_option('wpsc_s3_is_correct', 0);
	if(isset($wpsc_s3_access_key) & isset($wpsc_s3_secret_key) & isset($wpsc_s3_settings)){
		$s3 = new S3($wpsc_s3_access_key, $wpsc_s3_secret_key,false);
		$result = $s3->listBuckets();
		if(is_array($result)){
			//test if suppplied bucket exists
			$bucket = $s3->getBucket($wpsc_s3_settings['bucket']);
			if($bucket){
				//bucket is good
				$yes_tick = esc_url(admin_url('images/yes.png'));
				echo ('<strong style="color: green">Amazon s3 is connected   <img src="'.$yes_tick.'" alt="tick" width="16" height="16"  /></strong>');
				// set all good option
				update_option('wpsc_s3_is_correct', 1);
				
			}else {
				//bucket dosent exist
				echo ('<strong style="color: red">This Bucket dosen\'t exist   <img src="'.esc_url(admin_url('images/no.png')).'" alt="tick" width="16" height="16"  /></strong>');
			}
		}else {
			// error details dont work or the s3 account has no buckets
			echo ('<strong style="color: red">The provided user name and password are incorrect <img src="'.esc_url(admin_url('images/no.png')).'" alt="tick" width="16" height="16"  /></strong>');
		}
	} else {
		//details havn't been set yet
		//echo ('<p>enter details/p>');
	}
	?>
	<p class="submit">
			<input type="submit" value="Save Changes" class="button-primary" />
		</p>
		<input type='hidden' name='wpsc_admin_action' value='wpsc-update-s3-settings' />
		<?php wp_nonce_field('wpsc-s3-settings', 'wpsc-s3-settings'); ?>
		

	</form>
</div>
<?php
} 
function wpsc_save_s3_settings() {

	global $wpdb;
	check_admin_referer('wpsc-s3-settings', 'wpsc-s3-settings');
	if( isset($_POST['access_key']) ) {
		update_option('wpsc_s3_access_key', $_POST['access_key']);
	}
	if( isset($_POST['secret_key']) ) {
 		update_option('wpsc_s3_secret_key', $_POST['secret_key']);
	}
	if(is_array($_POST['s3_settings'])) {
		update_option( 'wpsc_s3_settings', $_POST['s3_settings'] );
		$wpsc_s3_access_key = get_option('wpsc_s3_access_key');
		$wpsc_s3_secret_key = get_option('wpsc_s3_secret_key');
		$wpsc_s3_settings = get_option('wpsc_s3_settings');
		
	} ?>
<div class="updated settings-error">
	<p><strong>Settings saved.</strong></p>
</div>
<?php
}
if(isset($_REQUEST['wpsc_admin_action'] )){
if($_REQUEST['wpsc_admin_action'] == 'wpsc-update-s3-settings') {
	add_action('admin_init', 'wpsc_save_s3_settings');
}
}
add_action('admin_init', 'add_amazon_meta_box_javascript');

function add_amazon_meta_box_javascript(){
	add_meta_box( 'wpsc_wpsc_s3_meta_box', __('Amazon s3', 'wpsc'), 'wpsc_s3_meta_box', 'wpsc-product', 'normal', 'low' );
}
add_action('admin_head', 'loadAmazon');

function loadAmazon(){?>
	<script type="text/javascript" >
		
		jQuery(document).ready(function($) {
			jQuery('#wpsc_Amazon_s3_loading').css('display', 'none');
		
			jQuery('#getAms3').click(function(){
				jQuery('#getAms3').remove();
				jQuery('#wpsc_Amazon_s3_loading').css('display', 'block');
			
				var data = {
					action:'amazon_s3_interface_callback'
				 };
			
				jQuery.post(ajaxurl, data, function(response) {
					jQuery('#wpsc_wpsc_s3_meta_box .inside').append(response);
				});
			});
		});
	</script><?php 
}
function wpsc_s3_meta_box(){
	if(get_option('wpsc_s3_is_correct')){?>

		<input type="button" value="Get Your Amazon Files" id="getAms3" ></input>
		<div id="wpsc_Amazon_s3_loading"><p><img  src="<?php echo esc_url(admin_url('images/wpspin_light.gif'))?>" alt="loading gif" /> gathering Files from Amazon</p></div>
		<?php
	} else {
		echo('<p> please configure your Amazon s3 settings </p>' );
	}
}
add_action('wp_ajax_amazon_s3_interface_callback', 'amazon_s3_interface_callback');
function amazon_s3_interface_callback(){
	//global  $wpdb, $wpsc_product_defaults;
	$wpsc_s3_access_key = get_option('wpsc_s3_access_key');
	$wpsc_s3_secret_key = get_option('wpsc_s3_secret_key');
	$wpsc_s3_settings = get_option('wpsc_s3_settings');
		//check option for ssl
	$is_ssl = get_option('wpsc_force_ssl');
	
	$s3 = new S3($wpsc_s3_access_key, $wpsc_s3_secret_key,$is_ssl);
	$prospective_bucket = $wpsc_s3_settings['bucket'];
	$bucket_contents = $s3->getBucket($prospective_bucket);
	// current selection
	?><script type="text/javascript" >
		jQuery(document).ready(function($) {
			jQuery('#wpsc_Amazon_s3_loading').css('display', 'none');
			//send file names that are clicked into 'Files assoicated' window
			var  count= 0;
			var message = '<p id="message" style="color:green;fontStyle:italic">The following file(s) will be added when this product is updated</p>';
			jQuery('#amazon_s3_file_list :checkbox').click(function(){
				if(jQuery(this).is(':checked')){
					if(count==0){
						jQuery('#amazon_s3_selected_list').append(message);
					}
					jQuery('#amazon_s3_selected_list').append(jQuery(this).next().clone());
					// once moved it wraps it with <p> for styling purposes
					jQuery('#amazon_s3_selected_list :last').wrap(function(){return '<p>'   + '</p>'});	
					count++;
				}else{
					//remove label element from 'Files assoicated' window
					var t = jQuery(this).next().text();
					jQuery('#amazon_s3_selected_list label ').remove(':contains('+t+')');
					count--;
					if(count==0){
						jQuery('#amazon_s3_selected_list  #message').remove();
					}
				}
			});
		});
	</script><?php 

	// s3 Bucket
	$output .= "<strong>All Files from your Amazon S3 Bucket</strong>";
	$output .= "<div id='amazon_s3_file_list' class='ui-widget-content multiple-select select_product_file' >";
	$num = 0;
	
	foreach($bucket_contents as $value){
	 	foreach($value as $key2 => $value2){
	 		if($key2 == name){
	 			$num++;
	 			
				$output .= "<p " . ((($num % 2) > 0) ? '' : "class='alt'") . " id='select_product_file_row_without_num'>\n";
				$output .= "<input type='checkbox' name='wpsc_s3_files[]'  value='$value2' >";
				$output .= "<label >$value2 </label>\n";
				$output .= "</p>\n";
	 		}
	 	}
	}
	$output .= "</div>";
	$output .= "<script type='text/javascript'>\r\n";
	$output .= "var select_min_height = " . (25 * 3) . ";\r\n";
	$output .= "var select_max_height = " . (25 * ($num + 1)) . ";\r\n";
	$output .= "</script>";
	echo($output);
	//
	?>	<strong>Files to be added</strong>
	<div id="amazon_s3_selected_list" class='ui-widget-content multiple-select select_product_file'></div><?php
	//
	die();
}

function amazon_s3_save_meta_data_to_post($post_ID){
	if( isset($_POST['wpsc_s3_files']) ){
		foreach($_POST['wpsc_s3_files'] as $s3File){
			$new_product_file_post= array(
			'post_title' => $s3File,
			'post_status' => 'inherit', 
			'post_name' => $s3File,
			'post_parent' => $post_ID,
			'guid' => 's3 file', // check if this will work
			'post_type' => 'wpsc-product-file');
			// get the post just created
			$id = wp_insert_post( $new_product_file_post );
			update_product_meta($id, 's3_file_name',$s3File);
		}
	
	} else {
		return;
	}
}

add_action('post_updated', 'amazon_s3_save_meta_data_to_post', 10, 2);


// this function hooks into wpec and creates s3 download links for products with s3 post meta 
function wpsc_generate_temporary_s3_link($file_id) {
	$meta = get_product_meta($file_id,'s3_file_name',1);
	if($meta){
		$file = get_post($file_id);
		$s3_file = get_product_meta($file->post_parent, '_s3_file_name', true);
		$wpsc_s3_access_key = get_option('wpsc_s3_access_key');
  		$wpsc_s3_secret_key = get_option('wpsc_s3_secret_key');
		$wpsc_s3_settings = get_option('wpsc_s3_settings');
		$downloadables_bucket = $wpsc_s3_settings['bucket'];
		$ssl = get_option('wpsc_force_ssl');
		$s3 = new S3($wpsc_s3_access_key, $wpsc_s3_secret_key,false);
		$auth_url = $s3->getAuthenticatedURL($downloadables_bucket, $file->post_title, 30);
		$mime_type = get_mime_type($meta);
		$meta = str_replace(' ' , '_',$meta);
		//exit('<pre>' .$meta.'</pre>');
		header('Content-Type:'.$mime_type.''); 
		header( 'Content-Disposition: attachment; filename='.$meta.'' );
		readfile($auth_url);
		//header("Location: {$auth_url}");
		exit();
	}else{
		return false;
	}
}
add_action('wpsc_alter_download_action', 'wpsc_generate_temporary_s3_link');

function get_mime_type($filename) {
	$mime_types = array(
		'txt' => 'text/plain',
        'htm' => 'text/html',
        'html' => 'text/html',
        'php' => 'text/html',
        'css' => 'text/css',
        'js' => 'application/javascript',
        'json' => 'application/json',
        'xml' => 'application/xml',
        'swf' => 'application/x-shockwave-flash',
        'flv' => 'video/x-flv',

        // images
        'png' => 'image/png',
        'jpe' => 'image/jpeg',
        'jpeg' => 'image/jpeg',
        'jpg' => 'image/jpeg',
        'gif' => 'image/gif',
        'bmp' => 'image/bmp',
        'ico' => 'image/vnd.microsoft.icon',
        'tiff' => 'image/tiff',
        'tif' => 'image/tiff',
        'svg' => 'image/svg+xml',
        'svgz' => 'image/svg+xml',

        // archives
        'zip' => 'application/zip',
        'rar' => 'application/x-rar-compressed',
        'exe' => 'application/x-msdownload',
        'msi' => 'application/x-msdownload',
        'cab' => 'application/vnd.ms-cab-compressed',

        // audio/video
        'mp3' => 'audio/mpeg',
        'qt' => 'video/quicktime',
        'mov' => 'video/quicktime',

        // adobe
        'pdf' => 'application/pdf',
        'psd' => 'image/vnd.adobe.photoshop',
        'ai' => 'application/postscript',
        'eps' => 'application/postscript',
        'ps' => 'application/postscript',

        // ms office
        'doc' => 'application/msword',
        'rtf' => 'application/rtf',
        'xls' => 'application/vnd.ms-excel',
        'ppt' => 'application/vnd.ms-powerpoint',

        // open office
        'odt' => 'application/vnd.oasis.opendocument.text',
        'ods' => 'application/vnd.oasis.opendocument.spreadsheet',
    );

    $ext = strtolower(array_pop(explode('.',$filename)));
    if (array_key_exists($ext, $mime_types)) {
        return $mime_types[$ext];
    }
    elseif (function_exists('finfo_open')) {
        $finfo = finfo_open(FILEINFO_MIME);
        $mimetype = finfo_file($finfo, $filename);
        finfo_close($finfo);
        return $mimetype;
    }
    else {
        return 'application/octet-stream';
    }
}

